@@ -4,9 +4,10 @@ import pytest
 pytest.importorskip('numpy')
 
 from operator import add, sub
-from tempfile import mkdtemp
-import shutil
 import os
+import shutil
+from tempfile import mkdtemp
+import time
 
 from toolz import merge
 from toolz.curried import identity
@@ -18,6 +19,13 @@ from dask.array.core import *
 from dask.utils import raises, ignoring, tmpfile
 from dask.array.utils import assert_eq
 
+# temporary until numpy functions migrated
+try:
+    from numpy import nancumsum, nancumprod
+except ImportError:  # pragma: no cover
+    import dask.array.numpy_compat as npcompat
+    nancumsum = npcompat.nancumsum
+    nancumprod = npcompat.nancumprod
 
 inc = lambda x: x + 1
 
@@ -285,6 +293,17 @@ def test_concatenate():
     assert raises(ValueError, lambda: concatenate([a, b, c], axis=2))
 
 
+def test_concatenate_fixlen_strings():
+    x = np.array(['a', 'b', 'c'])
+    y = np.array(['aa', 'bb', 'cc'])
+
+    a = da.from_array(x, chunks=(2,))
+    b = da.from_array(y, chunks=(2,))
+
+    assert_eq(np.concatenate([x, y]),
+              da.concatenate([a, b]))
+
+
 def test_vstack():
     x = np.arange(5)
     y = np.ones(5)
@@ -859,6 +878,81 @@ def test_store():
     assert raises(ValueError, lambda: store([at, bt], [at, bt]))
 
 
+def test_store_compute_false():
+    d = da.ones((4, 4), chunks=(2, 2))
+    a, b = d + 1, d + 2
+
+    at = np.zeros(shape=(4, 4))
+    bt = np.zeros(shape=(4, 4))
+
+    v = store([a, b], [at, bt], compute=False)
+    assert (at == 0).all() and (bt == 0).all()
+    v.compute()
+    assert (at == 2).all() and (bt == 3).all()
+
+
+class ThreadSafetyError(Exception):
+    pass
+
+
+class NonthreadSafeStore(object):
+    def __init__(self):
+        self.in_use = False
+
+    def __setitem__(self, key, value):
+        if self.in_use:
+            raise ThreadSafetyError()
+        self.in_use = True
+        time.sleep(0.001)
+        self.in_use = False
+
+
+class ThreadSafeStore(object):
+    def __init__(self):
+        self.concurrent_uses = 0
+        self.max_concurrent_uses = 0
+
+    def __setitem__(self, key, value):
+        self.concurrent_uses += 1
+        self.max_concurrent_uses = max(self.concurrent_uses, self.max_concurrent_uses)
+        time.sleep(0.01)
+        self.concurrent_uses -= 1
+
+
+def test_store_locks():
+    _Lock = type(Lock())
+    d = da.ones((10, 10), chunks=(2, 2))
+    a, b = d + 1, d + 2
+
+    at = np.zeros(shape=(10, 10))
+    bt = np.zeros(shape=(10, 10))
+
+    lock = Lock()
+    v = store([a, b], [at, bt], compute=False, lock=lock)
+    dsk = v.dask
+    locks = set(vv for v in dsk.values() for vv in v if isinstance(vv, _Lock))
+    assert locks == set([lock])
+
+    # Ensure same lock applies over multiple stores
+    at = NonthreadSafeStore()
+    v = store([a, b], [at, at], lock=lock,
+              get=dask.threaded.get, num_workers=10)
+
+    # Don't assume thread safety by default
+    at = NonthreadSafeStore()
+    store(a, at, get=dask.threaded.get, num_workers=10)
+    a.store(at, get=dask.threaded.get, num_workers=10)
+
+    # Ensure locks can be removed
+    at = ThreadSafeStore()
+    for i in range(10):
+        a.store(at, lock=False, get=dask.threaded.get, num_workers=10)
+        if at.max_concurrent_uses > 1:
+            break
+        if i == 9:
+            assert False
+
+
 def test_to_hdf5():
     try:
         import h5py
@@ -967,6 +1061,8 @@ def test_astype():
     assert_eq(d.astype('i8'), x.astype('i8'))
     assert same_keys(d.astype('i8'), d.astype('i8'))
 
+    assert d.astype(d.dtype) is d
+
 
 def test_arithmetic():
     x = np.arange(5).astype('f4') + 2
@@ -1773,10 +1869,19 @@ def test_to_imperative():
 
 def test_cumulative():
     x = da.arange(20, chunks=5)
-    y = x.cumsum(axis=0)
     assert_eq(x.cumsum(axis=0), np.arange(20).cumsum())
     assert_eq(x.cumprod(axis=0), np.arange(20).cumprod())
 
+    assert_eq(da.nancumsum(x, axis=0), nancumsum(np.arange(20)))
+    assert_eq(da.nancumprod(x, axis=0), nancumprod(np.arange(20)))
+
+    a = np.random.random((20))
+    rs = np.random.RandomState(0)
+    a[rs.rand(*a.shape) < 0.5] = np.nan
+    x = da.from_array(a, chunks=5)
+    assert_eq(da.nancumsum(x, axis=0), nancumsum(a))
+    assert_eq(da.nancumprod(x, axis=0), nancumprod(a))
+
     a = np.random.random((20, 24))
     x = da.from_array(a, chunks=(6, 5))
     assert_eq(x.cumsum(axis=0), a.cumsum(axis=0))
@@ -1784,12 +1889,38 @@ def test_cumulative():
     assert_eq(x.cumprod(axis=0), a.cumprod(axis=0))
     assert_eq(x.cumprod(axis=1), a.cumprod(axis=1))
 
+    assert_eq(da.nancumsum(x, axis=0), nancumsum(a, axis=0))
+    assert_eq(da.nancumsum(x, axis=1), nancumsum(a, axis=1))
+    assert_eq(da.nancumprod(x, axis=0), nancumprod(a, axis=0))
+    assert_eq(da.nancumprod(x, axis=1), nancumprod(a, axis=1))
+
+    a = np.random.random((20, 24))
+    rs = np.random.RandomState(0)
+    a[rs.rand(*a.shape) < 0.5] = np.nan
+    x = da.from_array(a, chunks=(6, 5))
+    assert_eq(da.nancumsum(x, axis=0), nancumsum(a, axis=0))
+    assert_eq(da.nancumsum(x, axis=1), nancumsum(a, axis=1))
+    assert_eq(da.nancumprod(x, axis=0), nancumprod(a, axis=0))
+    assert_eq(da.nancumprod(x, axis=1), nancumprod(a, axis=1))
+
     a = np.random.random((20, 24, 13))
     x = da.from_array(a, chunks=(6, 5, 4))
     for axis in [0, 1, 2]:
         assert_eq(x.cumsum(axis=axis), a.cumsum(axis=axis))
         assert_eq(x.cumprod(axis=axis), a.cumprod(axis=axis))
 
+        assert_eq(da.nancumsum(x, axis=axis), nancumsum(a, axis=axis))
+        assert_eq(da.nancumprod(x, axis=axis), nancumprod(a, axis=axis))
+
+    a = np.random.random((20, 24, 13))
+    rs = np.random.RandomState(0)
+    a[rs.rand(*a.shape) < 0.5] = np.nan
+    x = da.from_array(a, chunks=(6, 5, 4))
+    for axis in [0, 1, 2]:
+        assert_eq(da.nancumsum(x, axis=axis), nancumsum(a, axis=axis))
+        assert_eq(da.nancumprod(x, axis=axis), nancumprod(a, axis=axis))
+
+
 
 def test_eye():
     assert_eq(da.eye(9, chunks=3), np.eye(9))
@@ -1859,3 +1990,9 @@ def test_tril_triu_errors():
     dA = da.from_array(A, chunks=(5, 5))
     assert raises(NotImplementedError, lambda: da.triu(dA))
 
+
+def test_atop_names():
+    x = da.ones(5, chunks=(2,))
+    y = atop(add, 'i', x, 'i')
+    assert y.name.startswith('add')
+
