@@ -95,7 +95,7 @@ def optimize(dsk, keys, **kwargs):
 
 
 def to_textfiles(b, path, name_function=str, compression='infer',
-                 encoding=system_encoding):
+                 encoding=system_encoding, compute=True):
     """ Write bag to disk, one filename per partition, one line per element
 
     **Paths**: This will create one file for each partition in your bag. You
@@ -166,7 +166,11 @@ def to_textfiles(b, path, name_function=str, compression='infer',
                             encoding))
                for i, path in enumerate(paths))
 
-    return Bag(merge(b.dask, dsk), name, b.npartitions)
+    result = Bag(merge(b.dask, dsk), name, b.npartitions)
+    if compute:
+        result.compute()
+    else:
+        return result
 
 
 def finalize(results):
@@ -183,11 +187,55 @@ def finalize_item(results):
     return results[0]
 
 
+def unpack_kwargs(kwargs):
+    """ Extracts dask values from kwargs
+
+    Currently only dask.bag.Item and python literal values are supported.
+
+    Returns a merged dask graph and a list of [key, val] pairs suitable for
+    eventually constructing a dict.
+    """
+    dsk = {}
+    kw_pairs = []
+    for key, val in iteritems(kwargs):
+        if isinstance(val, Item):
+            dsk.update(val.dask)
+            val = val.key
+        # TODO elif isinstance(val, Value):
+        elif isinstance(val, Base):
+            raise NotImplementedError(
+                '%s not supported as kwarg value to Bag.map_partitions'
+                % type(val).__name__)
+        kw_pairs.append([key, val])
+    return dsk, kw_pairs
+
+
 class Item(Base):
     _optimize = staticmethod(optimize)
     _default_get = staticmethod(mpget)
     _finalize = staticmethod(finalize_item)
 
+    @staticmethod
+    def from_imperative(value):
+        """ Create bag item from an imperative value
+
+        Parameters
+        ----------
+        value: a Value
+            A single dask.imperative.Value object, such as come from dask.do
+
+        Returns
+        -------
+        Item
+
+        Examples
+        --------
+        >>> b = db.Item.from_imperative(x)  # doctest: +SKIP
+        """
+        from dask.imperative import Value
+        assert isinstance(value, Value)
+        return Item(value.dask, value.key)
+
     def __init__(self, dsk, key):
         self.dask = dsk
         self.key = key
@@ -203,6 +251,14 @@ class Item(Base):
 
     __int__ = __float__ = __complex__ = __bool__ = Base.compute
 
+    def to_imperative(self):
+        """ Convert bag item to dask Value
+
+        Returns a single value.
+        """
+        from dask.imperative import Value
+        return Value(self.key, [self.dask])
+
 
 class Bag(Base):
     """ Parallel collection of Python objects
@@ -244,20 +300,53 @@ class Bag(Base):
         self.npartitions = npartitions
         self.str = StringAccessor(self)
 
-    def map(self, func):
+    def map(self, func, **kwargs):
         """ Map a function across all elements in collection
 
         >>> import dask.bag as db
         >>> b = db.from_sequence(range(5))
         >>> list(b.map(lambda x: x * 10))  # doctest: +SKIP
         [0, 10, 20, 30, 40]
+
+        Keyword arguments are passed through to `func`. These can be either
+        `dask.bag.Item`s, or normal python objects.
+
+        Example
+        -------
+        >>> import dask.bag as db
+        >>> b = db.from_sequence(range(1, 101), npartitions=10)
+        >>> def div(num, den=1):
+        ...     return num / den
+
+        Using a python object:
+
+        >>> hi = b.max().compute()
+        >>> hi
+        100
+        >>> b.map(div, den=hi).take(5)
+        (0.01, 0.02, 0.03, 0.04, 0.05)
+
+        Using an `Item`:
+
+        >>> b.map(div, den=b.max()).take(5)
+        (0.01, 0.02, 0.03, 0.04, 0.05)
+
+        Note that while both versions give the same output, the second forms a
+        single graph, and then computes everything at once, and in some cases
+        may be more efficient.
         """
-        name = 'map-{0}-{1}'.format(funcname(func), tokenize(self, func))
+        name = 'map-{0}-{1}'.format(funcname(func),
+                                    tokenize(self, func, kwargs))
         if takes_multiple_arguments(func):
             func = partial(apply, func)
-        dsk = dict(((name, i), (reify, (map, func, (self.name, i))))
+        dsk = self.dask.copy()
+        if kwargs:
+            kw_dsk, kw_pairs = unpack_kwargs(kwargs)
+            dsk.update(kw_dsk)
+            func = (apply, partial, [func], (dict, kw_pairs))
+        dsk.update(((name, i), (reify, (map, func, (self.name, i))))
                    for i in range(self.npartitions))
-        return type(self)(merge(self.dask, dsk), name, self.npartitions)
+        return type(self)(dsk, name, self.npartitions)
 
     @property
     def _args(self):
@@ -297,19 +386,52 @@ class Bag(Base):
                    for i in range(self.npartitions))
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
-    def map_partitions(self, func):
+    def map_partitions(self, func, **kwargs):
         """ Apply function to every partition within collection
 
         Note that this requires you to understand how dask.bag partitions your
         data and so is somewhat internal.
 
         >>> b.map_partitions(myfunc)  # doctest: +SKIP
+
+        Keyword arguments are passed through to `func`. These can be either
+        `dask.bag.Item`s, or normal python objects.
+
+        Example
+        -------
+        >>> import dask.bag as db
+        >>> b = db.from_sequence(range(1, 101), npartitions=10)
+        >>> def div(nums, den=1):
+        ...     return [num / den for num in nums]
+
+        Using a python object:
+
+        >>> hi = b.max().compute()
+        >>> hi
+        100
+        >>> b.map_partitions(div, den=hi).take(5)
+        (0.01, 0.02, 0.03, 0.04, 0.05)
+
+        Using an `Item`:
+
+        >>> b.map_partitions(div, den=b.max()).take(5)
+        (0.01, 0.02, 0.03, 0.04, 0.05)
+
+        Note that while both versions give the same output, the second forms a
+        single graph, and then computes everything at once, and in some cases
+        may be more efficient.
         """
         name = 'map-partitions-{0}-{1}'.format(funcname(func),
-                                               tokenize(self, func))
-        dsk = dict(((name, i), (func, (self.name, i)))
+                                               tokenize(self, func, kwargs))
+        dsk = self.dask.copy()
+        if kwargs:
+            kw_dsk, kw_pairs = unpack_kwargs(kwargs)
+            dsk.update(kw_dsk)
+        dsk.update(((name, i),
+                    (apply, func, [(self.name, i)], (dict, kw_pairs))
+                    if kwargs else (func, (self.name, i)))
                    for i in range(self.npartitions))
-        return type(self)(merge(self.dask, dsk), name, self.npartitions)
+        return type(self)(dsk, name, self.npartitions)
 
     def pluck(self, key, default=no_default):
         """ Select item from all tuples/dicts in collection
@@ -343,8 +465,8 @@ class Bag(Base):
 
     @wraps(to_textfiles)
     def to_textfiles(self, path, name_function=str, compression='infer',
-                     encoding=system_encoding):
-        return to_textfiles(self, path, name_function, compression, encoding)
+                     encoding=system_encoding, compute=True):
+        return to_textfiles(self, path, name_function, compression, encoding, compute)
 
     def fold(self, binop, combine=None, initial=no_default, split_every=None):
         """ Parallelizable reduction
@@ -946,9 +1068,23 @@ def write(data, filename, compression, encoding):
             os.makedirs(dirname)
 
     f = open(filename, mode='wb', compression=compression)
+
+    # Check presence of endlines
+    data = iter(data)
+    firstline = next(data)
+    if not (firstline.endswith(os.linesep) or firstline.endswith('\n')):
+        sep = os.linesep if firstline.endswith(os.linesep) else '\n'
+        firstline = firstline + sep
+        data = (line + sep for line in data)
+    f.write(firstline.encode(encoding))
+
     try:
+        lastline = ''
         for line in data:
-            f.write(line.encode(encoding))
+            f.write(lastline.encode(encoding))
+            lastline = line
+        f.write(lastline.rstrip(os.linesep).encode(encoding))
+
     finally:
         f.close()
 
