@@ -10,7 +10,7 @@ from dask.bag.core import (Bag, lazify, lazify_task, fuse, map, collect,
         reduceby, bz2_stream, stream_decompress, reify, partition,
         _parse_s3_URI, inline_singleton_lists, optimize, system_encoding,
         from_imperative)
-from dask.compatibility import BZ2File, GzipFile
+from dask.compatibility import BZ2File, GzipFile, reduce
 from dask.utils import filetexts, tmpfile, raises, open
 from dask.async import get_sync
 import dask
@@ -103,6 +103,17 @@ def test_map_with_builtins():
         [True, False]
 
 
+def test_map_with_kwargs():
+    b = db.from_sequence(range(100), npartitions=10)
+    assert b.map(lambda x, factor=0: x * factor,
+                 factor=2).sum().compute() == 9900.0
+    assert b.map(lambda x, total=0: x / total,
+                 total=b.sum()).sum().compute() == 1.0
+    assert b.map(lambda x, factor=0, total=0: x * factor / total,
+                 total=b.sum(),
+                 factor=2).sum().compute() == 2.0
+
+
 def test_filter():
     c = b.filter(iseven)
     expected = merge(dsk, dict(((c.name, i),
@@ -300,6 +311,20 @@ def test_map_partitions():
     assert b.map_partitions(lambda a: len(a) + 1).name != b.map_partitions(len).name
 
 
+def test_map_partitions_with_kwargs():
+    b = db.from_sequence(range(100), npartitions=10)
+    assert b.map_partitions(
+        lambda X, factor=0: [x * factor for x in X],
+        factor=2).sum().compute() == 9900.0
+    assert b.map_partitions(
+        lambda X, total=0: [x / total for x in X],
+        total=b.sum()).sum().compute() == 1.0
+    assert b.map_partitions(
+        lambda X, factor=0, total=0: [x * factor / total for x in X],
+        total=b.sum(),
+        factor=2).sum().compute() == 2.0
+
+
 def test_lazify_task():
     task = (sum, (reify, (map, inc, [1, 2, 3])))
     assert lazify_task(task) == (sum, (map, inc, [1, 2, 3]))
@@ -360,6 +385,7 @@ def test_can_use_dict_to_make_concrete():
     assert isinstance(dict(b.frequencies()), dict)
 
 
+@pytest.mark.xfail(reason="bloscpack BLOSC_MAX_BUFFERSIZE")
 def test_from_castra():
     castra = pytest.importorskip('castra')
     pd = pytest.importorskip('pandas')
@@ -649,7 +675,7 @@ def test_to_textfiles():
     b = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
     dir = mkdtemp()
     for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
-        c = b.to_textfiles(os.path.join(dir, '*.' + ext))
+        c = b.to_textfiles(os.path.join(dir, '*.' + ext), compute=False)
         assert c.npartitions == b.npartitions
         try:
             c.compute(get=dask.get)
@@ -670,7 +696,7 @@ def test_to_textfiles_encoding():
     b = db.from_sequence([u'汽车', u'苹果', u'天气'], npartitions=2)
     dir = mkdtemp()
     for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
-        c = b.to_textfiles(os.path.join(dir, '*.' + ext), encoding='gb18030')
+        c = b.to_textfiles(os.path.join(dir, '*.' + ext), encoding='gb18030', compute=False)
         assert c.npartitions == b.npartitions
         try:
             c.compute(get=dask.get)
@@ -691,17 +717,26 @@ def test_to_textfiles_inputs():
     B = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
     with tmpfile() as a:
         with tmpfile() as b:
-            B.to_textfiles([a, b]).compute()
+            B.to_textfiles([a, b])
             assert os.path.exists(a)
             assert os.path.exists(b)
 
     with tmpfile() as dirname:
-        B.to_textfiles(dirname).compute()
+        B.to_textfiles(dirname)
         assert os.path.exists(dirname)
         assert os.path.exists(os.path.join(dirname, '0.part'))
     assert raises(ValueError, lambda: B.to_textfiles(5))
 
 
+def test_to_textfiles_endlines():
+    b = db.from_sequence(['a', 'b', 'c'], npartitions=1)
+    with tmpfile() as fn:
+        b.to_textfiles([fn])
+        with open(fn, 'r') as f:
+            result = f.readlines()
+        assert result == ['a\n', 'b\n', 'c']
+
+
 def test_bz2_stream():
     text = '\n'.join(map(str, range(10000)))
     compressed = bz2.compress(text.encode())
@@ -810,14 +845,20 @@ def test_bag_compute_forward_kwargs():
 
 def test_to_imperative():
     from dask.imperative import Value
+
     b = db.from_sequence([1, 2, 3, 4, 5, 6], npartitions=3)
     a, b, c = b.map(inc).to_imperative()
     assert all(isinstance(x, Value) for x in [a, b, c])
     assert b.compute() == [4, 5]
 
+    b = db.from_sequence([1, 2, 3, 4, 5, 6], npartitions=3)
+    t = b.sum().to_imperative()
+    assert isinstance(t, Value)
+    assert t.compute() == 21
+
 
 def test_from_imperative():
-    from dask.imperative import value
+    from dask.imperative import value, do
     a, b, c = value([1, 2, 3]), value([4, 5, 6]), value([7, 8, 9])
     bb = from_imperative([a, b, c])
     assert bb.name == from_imperative([a, b, c]).name
@@ -825,6 +866,10 @@ def test_from_imperative():
     assert isinstance(bb, Bag)
     assert list(bb) == [1, 2, 3, 4, 5, 6, 7, 8, 9]
 
+    asum_value = do(lambda X: sum(X))(a)
+    asum_item = db.Item.from_imperative(asum_value)
+    assert asum_value.compute() == asum_item.compute() == 6
+
 
 def test_range():
     for npartitions in [1, 7, 10, 28]:
