@@ -91,8 +91,12 @@ class Scalar(Base):
     def _args(self):
         return (self.dask, self._name)
 
+    @property
+    def key(self):
+        return (self._name, 0)
+
     def _keys(self):
-        return [(self._name, 0)]
+        return [self.key]
 
     @classmethod
     def _get_unary_operator(cls, op):
@@ -180,13 +184,6 @@ class _Frame(Base):
         """Return number of partitions"""
         return len(self.divisions) - 1
 
-    @property
-    def _pd(self):
-        """
-        Return an empty pd.DataFrame / pd.Series to emulate calculation result.
-        """
-        return self._pd_cache
-
     @classmethod
     def _build_pd(cls, metadata):
         """ build pandas instance from passed metadata """
@@ -351,7 +348,7 @@ class _Frame(Base):
     def head(self, n=5, compute=True):
         """ First n rows of the dataset
 
-        Caveat, the only checks the first n rows of the first partition.
+        Caveat, this only checks the first n rows of the first partition.
         """
         name = 'head-%d-%s' % (n, self._name)
         dsk = {(name, 0): (lambda x, n: x.head(n=n), (self._name, 0), n)}
@@ -887,7 +884,7 @@ class Series(_Frame):
         result.dask = dsk
         result._name = _name
 
-        result._pd_cache, result._known_dtype = cls._build_pd(name)
+        result._pd, result._known_dtype = cls._build_pd(name)
 
         result.divisions = tuple(divisions)
         return result
@@ -927,8 +924,8 @@ class Series(_Frame):
         if self._known_dtype:
             return self.dtype
         else:
-            self._pd_cache, self._known_dtype = self._build_pd(self.head())
-            return self._pd_cache.dtype
+            self._pd, self._known_dtype = self._build_pd(self.head())
+            return self._pd.dtype
 
     def __getattr__(self, key):
         if key == 'cat':
@@ -1135,6 +1132,10 @@ class Series(_Frame):
     def notnull(self):
         return self.map_partitions(pd.Series.notnull)
 
+    @derived_from(pd.Series)
+    def isnull(self):
+        return self.map_partitions(pd.Series.isnull)
+
     def to_bag(self, index=False):
         """Convert to a dask Bag.
 
@@ -1197,10 +1198,10 @@ class Series(_Frame):
 
         if name is no_default:
             msg = ("name is not specified, inferred from partial data. "
-                   "Please provide name if the result is unecpected.\n"
-                   "  Before: s.apply(func)\n"
-                   "  After:  s.apply(func, name=['x', 'y']) for dataframe result\n"
-                   "  or:     s.apply(func, name='x')        for series result")
+                   "Please provide name if the result is unexpected.\n"
+                   "  Before: .apply(func)\n"
+                   "  After:  .apply(func, name=['x', 'y']) for dataframe result\n"
+                   "  or:     .apply(func, name='x')        for series result")
             warnings.warn(msg)
 
             name = _emulate(pd.Series.apply, self.head(), func,
@@ -1244,6 +1245,21 @@ class Index(Series):
     def _constructor(self):
         return Index
 
+    def head(self, n=5, compute=True):
+        """ First n items of the Index.
+
+        Caveat, this only checks the first partition.
+        """
+        name = 'head-%d-%s' % (n, self._name)
+        dsk = {(name, 0): (lambda x, n: x[:n], (self._name, 0), n)}
+
+        result = self._constructor(merge(self.dask, dsk), name,
+                                   self._pd, self.divisions[:2])
+
+        if compute:
+            result = result.compute()
+        return result
+
     def nunique(self):
         return self.drop_duplicates().count()
 
@@ -1287,7 +1303,7 @@ class DataFrame(_Frame):
         result.dask = dask
         result._name = name
 
-        result._pd_cache, result._known_dtype = cls._build_pd(columns)
+        result._pd, result._known_dtype = cls._build_pd(columns)
         result.divisions = tuple(divisions)
         return result
 
@@ -1378,8 +1394,8 @@ class DataFrame(_Frame):
         if self._known_dtype:
             return self._pd.dtypes
         else:
-            self._pd_cache, self._known_dtype = self._build_pd(self.head())
-            return self._pd_cache.dtypes
+            self._pd, self._known_dtype = self._build_pd(self.head())
+            return self._pd.dtypes
 
     @derived_from(pd.DataFrame)
     def set_index(self, other, drop=True, **kwargs):
@@ -1502,7 +1518,9 @@ class DataFrame(_Frame):
         return to_bag(self, index)
 
     def _get_numeric_data(self, how='any', subset=None):
-
+        # If unknown dtype, need to infer from head.
+        if not self._known_dtype:
+            self.dtypes
         # calculate columns to avoid unnecessary calculation
         numerics = self._pd._get_numeric_data()
 
@@ -1656,10 +1674,10 @@ class DataFrame(_Frame):
 
         if columns is no_default:
             msg = ("columns is not specified, inferred from partial data. "
-                   "Please provide columns if the result is unecpected.\n"
-                   "  Before: df.apply(func)\n"
-                   "  After:  df.apply(func, columns=['x', 'y']) for dataframe result\n"
-                   "  or:     df.apply(func, columns='x')        for series result")
+                   "Please provide columns if the result is unexpected.\n"
+                   "  Before: .apply(func)\n"
+                   "  After:  .apply(func, columns=['x', 'y']) for dataframe result\n"
+                   "  or:     .apply(func, columns='x')        for series result")
             warnings.warn(msg)
 
             columns = _emulate(pd.DataFrame.apply, self.head(), func,
@@ -1679,6 +1697,11 @@ class DataFrame(_Frame):
                                       "implemented")
         return cov_corr(self, min_periods, True)
 
+    @derived_from(pd.DataFrame)
+    def astype(self, dtype):
+        empty = self._pd.astype(dtype)
+        return map_partitions(pd.DataFrame.astype, empty, self, dtype=dtype)
+
 
 # bind operators
 for op in [operator.abs, operator.add, operator.and_, operator_div,
@@ -2250,18 +2273,21 @@ def cov_corr_agg(data, meta, min_periods=2, corr=False, scalar=False):
     sums = np.nan_to_num(data['sum'])
     counts = data['count']
 
-    totals = np.cumsum(sums, 0)
-    ns = np.cumsum(counts, 0)
+    cum_sums = np.cumsum(sums, 0)
+    cum_counts = np.cumsum(counts, 0)
 
-    ts = ns[:-1] * sums[1:] - counts[1:] * totals[:-1]
-    dens = ns[1:] * ns[:-1] * counts[1:]
-    dens = np.where(dens, dens, np.nan)
-    C = (np.nansum(data['cov'], 0) +
-         np.nansum(ts * ts.transpose((0, 2, 1)) / dens, 0))
-    C[ns[-1] < min_periods] = np.nan
-    nobs = np.where(ns[-1], ns[-1], np.nan)
+    s1 = cum_sums[:-1]
+    s2 = sums[1:]
+    n1 = cum_counts[:-1]
+    n2 = counts[1:]
+    d = (s2/n2) - (s1/n1)
+    C = (np.nansum((n1 * n2)/(n1 + n2) * (d * d.transpose((0, 2, 1))), 0) +
+         np.nansum(data['cov'], 0))
+
+    C[cum_counts[-1] < min_periods] = np.nan
+    nobs = np.where(cum_counts[-1], cum_counts[-1], np.nan)
     if corr:
-        mu = totals[-1] / nobs
+        mu = cum_sums[-1] / nobs
         counts_na = np.where(counts, counts, np.nan)
         m2 = np.nansum(data['m'] + counts*(sums/counts_na - mu)**2, axis=0)
         den = np.sqrt(m2 * m2.T)
