@@ -312,8 +312,7 @@ def from_array(x, chunksize=50000, columns=None):
     dummy = _dummy_from_array(x, columns)
 
     divisions = tuple(range(0, len(x), chunksize))
-    if divisions[-1] != len(x) - 1:
-        divisions = divisions + (len(x) - 1,)
+    divisions = divisions + (len(x) - 1,)
     token = tokenize(x, chunksize, columns)
     name = 'from_array-' + token
 
@@ -565,11 +564,11 @@ def from_dask_array(x, columns=None):
     >>> x = da.ones((4, 2), chunks=(2, 2))
     >>> df = dd.io.from_dask_array(x, columns=['a', 'b'])
     >>> df.compute()
-       a  b
-    0  1  1
-    1  1  1
-    2  1  1
-    3  1  1
+         a    b
+    0  1.0  1.0
+    1  1.0  1.0
+    2  1.0  1.0
+    3  1.0  1.0
     """
 
     dummy = _dummy_from_array(x, columns)
