@@ -72,6 +72,11 @@ def test_head_tail():
            sorted(d.tail(3, compute=False).dask)
 
 
+def test_index_head():
+    assert eq(d.index.head(2), full.index[:2])
+    assert eq(d.index.head(3), full.index[:3])
+
+
 def test_Series():
     assert isinstance(d.a, dd.Series)
     assert isinstance(d.a + 1, dd.Series)
@@ -1177,6 +1182,7 @@ def test_embarrassingly_parallel_operations():
     assert eq(a.x.clip(2, 4), df.x.clip(2, 4))
 
     assert eq(a.x.notnull(), df.x.notnull())
+    assert eq(a.x.isnull(), df.x.isnull())
 
     assert len(a.sample(0.5).compute()) < len(df)
 
@@ -1453,6 +1459,13 @@ def test_corr():
     pytest.raises(TypeError, lambda: da.corr(ddf))
 
 
+def test_cov_corr_stable():
+    df = pd.DataFrame(np.random.random((20000000, 2)) * 2 - 1, columns=['a', 'b'])
+    ddf = dd.from_pandas(df, npartitions=50)
+    assert eq(ddf.cov(), df.cov())
+    assert eq(ddf.corr(), df.corr())
+
+
 def test_apply_infer_columns():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
@@ -1589,3 +1602,12 @@ def test_to_imperative():
     assert isinstance(b, Value)
 
     assert eq(a.compute(), df.iloc[:2])
+
+
+def test_astype():
+    df = pd.DataFrame({'x': [1, 2, 3, None], 'y': [10, 20, 30, 40]},
+                      index=[10, 20, 30, 40])
+    a = dd.from_pandas(df, 2)
+
+    assert eq(a.astype(float), df.astype(float))
+    assert eq(a.x.astype(float), df.x.astype(float))
