@@ -3,14 +3,16 @@ import os
 import numpy as np
 import pytest
 
-from dask.compatibility import BZ2File, GzipFile
+from dask.compatibility import BZ2File, GzipFile, LZMAFile
 from dask.utils import (textblock, filetext, takes_multiple_arguments,
                         Dispatch, tmpfile, different_seeds)
 
 
 @pytest.mark.parametrize('myopen,compression',
-                         [(open, None), (GzipFile, 'gzip'), (BZ2File, 'bz2')])
+                         [(open, None), (GzipFile, 'gzip'), (BZ2File, 'bz2'), (LZMAFile, 'xz')])
 def test_textblock(myopen, compression):
+    if compression == 'xz':
+        pytest.importorskip('lzma')
     text = b'123 456 789 abc def ghi'.replace(b' ', os.linesep.encode())
     with filetext(text, open=myopen, mode='wb') as fn:
         text = ''.join(textblock(fn, 1, 11, compression)).encode()
